from datetime import datetime
import json
import psycopg2
import serial.tools.list_ports

def readline(self):
    # Функция читает serial буфер
    i = self.buf.find(b"\n")
    if i >= 0:
        r = self.buf[:i+1]
        self.buf = self.buf[i+1:]
        return r
    while True:
        i = max(1, min(2048, self.s.in_waiting))
        data = self.s.read(i)
        i = data.find(b"\n")
        if i >= 0:
            r = self.buf + data[:i+1]
            self.buf[0:] = data[i+1:]
            return r
        else:
            self.buf.extend(data)

def get_ports():
    # Функция возвращает список Com портов
    ports = serial.tools.list_ports.comports()
    return ports

def findArduino(portsFound):
    # Функция возвращает имя Com порта с ардуино
    commPort = 'None'
    numConnection = len(portsFound)
    
    for i in range(0,numConnection):
        port = portsFound[i]
        strPort = str(port)

        if 'Arduino' or 'CH340' in strPort: 
            splitPort = strPort.split(' ')
            commPort = (splitPort[0])
        return commPort

def get_typeStringtoPostgres(value):
    # Функция определяет тип данных для Postgres из текста
    try:
        if datetime.strptime(value or "", "%Y-%m-%dT%H:%M:%S"):
            return " timestamp"
    except ValueError:
        try:
            int(value)
            return" integer"
        except ValueError:
            try:
                float(value)
                return " real"
            except ValueError:
                return " text"

def get_NameJSONobject(dict_JSON):
    # Функция возвращает имя обьекта JSON
    return str(list(dict_JSON.keys())[0])

def get_JSONtoSQLcolumn(dict_JSON):
    # Функция создает строку для создания колонок SQL, исходя из ключей и значений входного словаря
    strColumn = ''
    for key, value in dict_JSON.items():
        strColumn = strColumn + key + get_typeStringtoPostgres(value) + ","
    return strColumn[0:-1]

def get_SQLCreateTable(dict_JSON):
    # Функция создает строку для создания таблицы SQL, исходя из ключей и значений входного словаря
    if len(dict_JSON) == 1:
       return "create table if not exists " + get_NameJSONobject(dict_JSON) + "(id SERIAL PRIMARY KEY, DatetimeRec timestamp," + get_JSONtoSQLcolumn(dict_JSON[get_NameJSONobject(dict_JSON)]) + ")"
    else:
        print("Не верный формат JSON. Не удалось прочитать обьект")

def get_JSONtoSQLValue(dict_JSON):
    # Функция создает строку перебирая данные, исходя из ключей и значений входного словаря
    strValue = ''
    for key,value in dict_JSON.items():
        strValue = strValue + "'" + value + "',"
    return strValue[0:-1]

def get_JSONtoSQLkey(dict_JSON):
    # Функция создает строку из названия столбцов, исходя из ключей и значений входного словаря
    strkey = ''
    for key,value in dict_JSON.items():
        strkey = strkey + key + ","
    return strkey[0:-1]

def get_SQLinsertvalue(dict_JSON):
    # Функция создает строку для передачи данных в таблицу SQL, исходя из ключей и значений входного словаря
    return "insert into " + list(dict_JSON.keys())[0] + "(DatetimeRec,"+ get_JSONtoSQLkey(dict_JSON[get_NameJSONobject(dict_JSON)])+") VALUES ('"+ str(datetime.now()) + "'," + get_JSONtoSQLValue(dict_JSON[get_NameJSONobject(dict_JSON)]) + ")"

def is_json(myjson):
    # Функция проверяет данные на формат JSON
    try:
        json_object = json.loads(myjson)
    except ValueError as e:
        return False
    return True

#Найдем порт с ардуино
connectPort = findArduino(get_ports())

#Подключимся к нему
if connectPort != 'None':
    ser = serial.Serial(connectPort,baudrate = 9600, timeout=1)
    #print('Присоединились к порту ' + connectPort)
else:
    print('Ошибка соединения к порту')

# Читаем и записываем буфер
while True:
    in_JSON = ser.readline()

    if is_json(in_JSON):
        dict_JSON = json.loads(in_JSON)

        try:
            #connection = psycopg2.connect(database="SmartHouse", user="sm", password="w7A8t", host="localhost", port="5432")
            connection = psycopg2.connect(database="SmartHouse", user="sm", password="w7A8t", host="192.168.1.112", port="5432")
            cursor = connection.cursor()
            # Создаем таблицу с именем обьекта JSON, если ее нет
            cursor.execute(get_SQLCreateTable(dict_JSON))
            # Записываем данныев таблицу с именем обьекта JSON
            cursor.execute(get_SQLinsertvalue(dict_JSON))
            connection.commit()
            count = cursor.rowcount
            #print(count, " запись успешно добавлена!")

        except (Exception, psycopg2.Error) as error:
            print("Ошибка записи в таблицу", error)

        finally:
            if connection:
                cursor.close()
                connection.close()
                #print("Соединение закрыто")
